// Unreal Fighting Framework by Kevin Poretti

#pragma once

// FF includes
#include "FFInputBufferComponent.h"
#include "IFFSystemInterface.h"
#include "Utils/TCircleBuffer.h"

// UE includes
#include "CoreMinimal.h"
#include "GameFramework/PlayerController.h"
#include "InputMappingContext.h"

#include "FFPlayerController.generated.h"

/**
 *	A class that collects player inputs, stores them in an input buffer, and sends a rolling window of
 *	unacknowledged inputs to a remote client or server for processing.
 */
UCLASS()
class UNREALFIGHTINGFRAMEWORK_API AFFPlayerController : public APlayerController, public IFFSystemInterface
{
	GENERATED_BODY()

public:
	AFFPlayerController();

	/**
	 *	Sends all unacknowledged inputs to the remote
	 */
	virtual void SendInputsToRemote() const;

	// IFFSystemInterface interface
	virtual void FixedTick(float OneFrame) override;
	// End of IFFSystemInterface

	// APlayerController interface
	virtual void SetupInputComponent() override;
	// End of APlayerController interface

protected:
    /** MappingContext */
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category = "UFF|Input", meta = (AllowPrivateAccess = "true"))
	TSoftObjectPtr<UInputMappingContext> DefaultInputMapping;

	/** Input Buffer component */
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "UFF|Input", meta = (AllowPrivateAccess = "true"))
	UFFInputBufferComponent* InputBuffer;

	/** Current state of the player's inputs */
	FFFInputState CurrInput;

	/**
	 *	Rolling window of the player's past inputs that have yet to be
	 *	acknowledged and simulated by the remote machine
	 *
	 *	This ring buffer should be initialized to be the size of the past X frames
	 *	you want the remote machine to re-simulate, where X is the oldest input you want to
	 *	allow to be re-simulated.
	 */
	TCircleBuffer<FFFInputState, 20> UnacknowledgedInputs;
};
